@@ -52,7 +52,7 @@ GST_DEBUG_CATEGORY_STATIC (deinterlace_debug);
 /* Properties */
 
 #define DEFAULT_MODE            GST_DEINTERLACE_MODE_AUTO
-#define DEFAULT_METHOD          GST_DEINTERLACE_GREEDY_H
+#define DEFAULT_METHOD          GST_DEINTERLACE_LINEAR
 #define DEFAULT_FIELDS          GST_DEINTERLACE_ALL
 #define DEFAULT_FIELD_LAYOUT    GST_DEINTERLACE_LAYOUT_AUTO
 
@@ -369,7 +369,7 @@ gst_deinterlace_base_init (gpointer klass)
 
   gst_element_class_set_details_simple (element_class,
       "Deinterlacer",
-      "Filter/Video",
+      "Filter/Effect/Video/Deinterlace",
       "Deinterlace Methods ported from DScaler/TvTime",
       "Martin Eikermann <meiker@upb.de>, "
       "Sebastian Dröge <sebastian.droege@collabora.co.uk>");
@@ -593,6 +593,16 @@ gst_deinterlace_reset_history (GstDeinterlace * self, gboolean drop_all)
 {
   gint i;
 
+  if (!drop_all) {
+    GST_DEBUG_OBJECT (self, "Flushing history (count %d)", self->history_count);
+    while (self->history_count > 0) {
+      if (gst_deinterlace_output_frame (self, TRUE) != GST_FLOW_OK) {
+        /* Encountered error, or flushing -> skip and drop all remaining */
+        drop_all = TRUE;
+        break;
+      }
+    }
+  }
   if (drop_all) {
     GST_DEBUG_OBJECT (self, "Resetting history (count %d)",
         self->history_count);
@@ -603,10 +613,6 @@ gst_deinterlace_reset_history (GstDeinterlace * self, gboolean drop_all)
         self->field_history[i].buf = NULL;
       }
     }
-  } else {
-    GST_DEBUG_OBJECT (self, "Flushing history (count %d)", self->history_count);
-    while (self->history_count > 0)
-      gst_deinterlace_output_frame (self, TRUE);
   }
   memset (self->field_history, 0,
       GST_DEINTERLACE_MAX_FIELD_HISTORY * sizeof (GstDeinterlaceField));
