@@ -42,12 +42,6 @@ enum
   PROP_LAST
 };
 
-static const GstElementDetails gst_rtp_g726_pay_details =
-GST_ELEMENT_DETAILS ("RTP G.726 payloader",
-    "Codec/Payloader/Network",
-    "Payload-encodes G.726 audio into a RTP packet",
-    "Axis Communications <dev-gstreamer@axis.com>");
-
 static GstStaticPadTemplate gst_rtp_g726_pay_sink_template =
 GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_SINK,
@@ -93,7 +87,10 @@ gst_rtp_g726_pay_base_init (gpointer klass)
       gst_static_pad_template_get (&gst_rtp_g726_pay_sink_template));
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&gst_rtp_g726_pay_src_template));
-  gst_element_class_set_details (element_class, &gst_rtp_g726_pay_details);
+  gst_element_class_set_details_simple (element_class, "RTP G.726 payloader",
+      "Codec/Payloader/Network/RTP",
+      "Payload-encodes G.726 audio into a RTP packet",
+      "Axis Communications <dev-gstreamer@axis.com>");
 }
 
 static void
@@ -418,5 +415,5 @@ gboolean
 gst_rtp_g726_pay_plugin_init (GstPlugin * plugin)
 {
   return gst_element_register (plugin, "rtpg726pay",
-      GST_RANK_NONE, GST_TYPE_RTP_G726_PAY);
+      GST_RANK_SECONDARY, GST_TYPE_RTP_G726_PAY);
 }
