@@ -29,13 +29,6 @@
 
 #include "gstrtppcmapay.h"
 
-/* elementfactory information */
-static const GstElementDetails gst_rtp_pcma_pay_details =
-GST_ELEMENT_DETAILS ("RTP PCMA payloader",
-    "Codec/Payloader/Network",
-    "Payload-encodes PCMA audio into a RTP packet",
-    "Edgard Lima <edgard.lima@indt.org.br>");
-
 static GstStaticPadTemplate gst_rtp_pcma_pay_sink_template =
 GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_SINK,
@@ -72,7 +65,10 @@ gst_rtp_pcma_pay_base_init (gpointer klass)
       gst_static_pad_template_get (&gst_rtp_pcma_pay_sink_template));
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&gst_rtp_pcma_pay_src_template));
-  gst_element_class_set_details (element_class, &gst_rtp_pcma_pay_details);
+  gst_element_class_set_details_simple (element_class, "RTP PCMA payloader",
+      "Codec/Payloader/Network/RTP",
+      "Payload-encodes PCMA audio into a RTP packet",
+      "Edgard Lima <edgard.lima@indt.org.br>");
 }
 
 static void
@@ -82,8 +78,6 @@ gst_rtp_pcma_pay_class_init (GstRtpPmcaPayClass * klass)
 
   gstbasertppayload_class = (GstBaseRTPPayloadClass *) klass;
 
-  parent_class = g_type_class_peek_parent (klass);
-
   gstbasertppayload_class->set_caps = gst_rtp_pcma_pay_setcaps;
 }
 
@@ -120,5 +114,5 @@ gboolean
 gst_rtp_pcma_pay_plugin_init (GstPlugin * plugin)
 {
   return gst_element_register (plugin, "rtppcmapay",
-      GST_RANK_NONE, GST_TYPE_RTP_PCMA_PAY);
+      GST_RANK_SECONDARY, GST_TYPE_RTP_PCMA_PAY);
 }
