@@ -78,7 +78,7 @@ gst_rtp_pcmu_depay_base_init (gpointer klass)
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&gst_rtp_pcmu_depay_sink_template));
   gst_element_class_set_details_simple (element_class, "RTP PCMU depayloader",
-      "Codec/Depayloader/Network",
+      "Codec/Depayloader/Network/RTP",
       "Extracts PCMU audio from RTP packets",
       "Edgard Lima <edgard.lima@indt.org.br>, Zeeshan Ali <zeenix@gmail.com>");
 }
@@ -158,5 +158,5 @@ gboolean
 gst_rtp_pcmu_depay_plugin_init (GstPlugin * plugin)
 {
   return gst_element_register (plugin, "rtppcmudepay",
-      GST_RANK_MARGINAL, GST_TYPE_RTP_PCMU_DEPAY);
+      GST_RANK_SECONDARY, GST_TYPE_RTP_PCMU_DEPAY);
 }
