@@ -199,7 +199,6 @@ enum
 #define DEFAULT_NUM_SOURCES          0
 #define DEFAULT_NUM_ACTIVE_SOURCES   0
 #define DEFAULT_USE_PIPELINE_CLOCK   FALSE
-#define DEFAULT_RTCP_MIN_INTERVAL    (RTP_STATS_MIN_INTERVAL * GST_SECOND)
 
 enum
 {
@@ -214,7 +213,6 @@ enum
   PROP_NUM_ACTIVE_SOURCES,
   PROP_INTERNAL_SESSION,
   PROP_USE_PIPELINE_CLOCK,
-  PROP_RTCP_MIN_INTERVAL,
   PROP_LAST
 };
 
@@ -257,10 +255,6 @@ static GstFlowReturn gst_rtp_session_sync_rtcp (RTPSession * sess,
 static gint gst_rtp_session_clock_rate (RTPSession * sess, guint8 payload,
     gpointer user_data);
 static void gst_rtp_session_reconsider (RTPSession * sess, gpointer user_data);
-static void gst_rtp_session_request_key_unit (RTPSession * sess,
-    gboolean all_headers, gpointer user_data);
-static GstClockTime gst_rtp_session_request_time (RTPSession * session,
-    gpointer user_data);
 
 static RTPSessionCallbacks callbacks = {
   gst_rtp_session_process_rtp,
@@ -268,9 +262,7 @@ static RTPSessionCallbacks callbacks = {
   gst_rtp_session_sync_rtcp,
   gst_rtp_session_send_rtcp,
   gst_rtp_session_clock_rate,
-  gst_rtp_session_reconsider,
-  gst_rtp_session_request_key_unit,
-  gst_rtp_session_request_time
+  gst_rtp_session_reconsider
 };
 
 /* GObject vmethods */
@@ -373,22 +365,22 @@ gst_rtp_session_base_init (gpointer klass)
   GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
 
   /* sink pads */
-  gst_element_class_add_static_pad_template (element_class,
-      &rtpsession_recv_rtp_sink_template);
-  gst_element_class_add_static_pad_template (element_class,
-      &rtpsession_recv_rtcp_sink_template);
-  gst_element_class_add_static_pad_template (element_class,
-      &rtpsession_send_rtp_sink_template);
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&rtpsession_recv_rtp_sink_template));
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&rtpsession_recv_rtcp_sink_template));
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&rtpsession_send_rtp_sink_template));
 
   /* src pads */
-  gst_element_class_add_static_pad_template (element_class,
-      &rtpsession_recv_rtp_src_template);
-  gst_element_class_add_static_pad_template (element_class,
-      &rtpsession_sync_src_template);
-  gst_element_class_add_static_pad_template (element_class,
-      &rtpsession_send_rtp_src_template);
-  gst_element_class_add_static_pad_template (element_class,
-      &rtpsession_send_rtcp_src_template);
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&rtpsession_recv_rtp_src_template));
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&rtpsession_sync_src_template));
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&rtpsession_send_rtp_src_template));
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&rtpsession_send_rtcp_src_template));
 
   gst_element_class_set_details_simple (element_class, "RTP Session",
       "Filter/Network/RTP",
@@ -596,12 +588,6 @@ gst_rtp_session_class_init (GstRtpSessionClass * klass)
           DEFAULT_USE_PIPELINE_CLOCK,
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
-  g_object_class_install_property (gobject_class, PROP_RTCP_MIN_INTERVAL,
-      g_param_spec_uint64 ("rtcp-min-interval", "Minimum RTCP interval",
-          "Minimum interval between Regular RTCP packet (in ns)",
-          0, G_MAXUINT64, DEFAULT_RTCP_MIN_INTERVAL,
-          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
-
   gstelement_class->change_state =
       GST_DEBUG_FUNCPTR (gst_rtp_session_change_state);
   gstelement_class->request_new_pad =
@@ -707,10 +693,6 @@ gst_rtp_session_set_property (GObject * object, guint prop_id,
     case PROP_USE_PIPELINE_CLOCK:
       priv->use_pipeline_clock = g_value_get_boolean (value);
       break;
-    case PROP_RTCP_MIN_INTERVAL:
-      g_object_set_property (G_OBJECT (priv->session), "rtcp-min-interval",
-          value);
-      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -763,10 +745,6 @@ gst_rtp_session_get_property (GObject * object, guint prop_id,
     case PROP_USE_PIPELINE_CLOCK:
       g_value_set_boolean (value, priv->use_pipeline_clock);
       break;
-    case PROP_RTCP_MIN_INTERVAL:
-      g_object_get_property (G_OBJECT (priv->session), "rtcp-min-interval",
-          value);
-      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -837,10 +815,6 @@ rtcp_thread (GstRtpSession * rtpsession)
 
   session = rtpsession->priv->session;
 
-  GST_DEBUG_OBJECT (rtpsession, "starting at %" GST_TIME_FORMAT,
-      GST_TIME_ARGS (current_time));
-  session->start_time = current_time;
-
   while (!rtpsession->priv->stop_thread) {
     GstClockReturn res;
 
@@ -1329,10 +1303,6 @@ gst_rtp_session_event_recv_rtp_sink (GstPad * pad, GstEvent * event)
   gboolean ret = FALSE;
 
   rtpsession = GST_RTP_SESSION (gst_pad_get_parent (pad));
-  if (G_UNLIKELY (rtpsession == NULL)) {
-    gst_event_unref (event);
-    return FALSE;
-  }
 
   GST_DEBUG_OBJECT (rtpsession, "received event %s",
       GST_EVENT_TYPE_NAME (event));
@@ -1384,112 +1354,27 @@ gst_rtp_session_event_recv_rtp_sink (GstPad * pad, GstEvent * event)
 
 }
 
-static gboolean
-gst_rtp_session_request_remote_key_unit (GstRtpSession * rtpsession,
-    guint32 ssrc, guint payload, gboolean all_headers, gint count)
-{
-  GstCaps *caps;
-
-  caps = gst_rtp_session_get_caps_for_pt (rtpsession, payload);
-
-  if (caps) {
-    const GstStructure *s = gst_caps_get_structure (caps, 0);
-    gboolean pli;
-    gboolean fir;
-
-    pli = gst_structure_has_field (s, "rtcp-fb-nack-pli");
-    fir = gst_structure_has_field (s, "rtcp-fb-ccm-fir") && all_headers;
-
-    /* Google Talk uses FIR for repair, so send it even if we just want a
-     * regular PLI */
-    if (!pli &&
-        gst_structure_has_field (s, "rtcp-fb-x-gstreamer-fir-as-repair"))
-      fir = TRUE;
-
-    gst_caps_unref (caps);
-
-    if (pli || fir)
-      return rtp_session_request_key_unit (rtpsession->priv->session, ssrc,
-          gst_clock_get_time (rtpsession->priv->sysclock), fir, count);
-  }
-
-  return FALSE;
-}
-
-static gboolean
-gst_rtp_session_event_recv_rtp_src (GstPad * pad, GstEvent * event)
-{
-  GstRtpSession *rtpsession;
-  gboolean forward = TRUE;
-  gboolean ret = TRUE;
-  const GstStructure *s;
-  guint32 ssrc;
-  guint pt;
-
-  rtpsession = GST_RTP_SESSION (gst_pad_get_parent (pad));
-  if (G_UNLIKELY (rtpsession == NULL)) {
-    gst_event_unref (event);
-    return FALSE;
-  }
-
-  switch (GST_EVENT_TYPE (event)) {
-    case GST_EVENT_CUSTOM_UPSTREAM:
-      s = gst_event_get_structure (event);
-      if (gst_structure_has_name (s, "GstForceKeyUnit") &&
-          gst_structure_get_uint (s, "ssrc", &ssrc) &&
-          gst_structure_get_uint (s, "payload", &pt)) {
-        gboolean all_headers = FALSE;
-        gint count = -1;
-
-        gst_structure_get_boolean (s, "all-headers", &all_headers);
-        if (gst_structure_get_int (s, "count", &count) && count < 0)
-          count += G_MAXINT;    /* Make sure count is positive if present */
-        if (gst_rtp_session_request_remote_key_unit (rtpsession, ssrc, pt,
-                all_headers, count))
-          forward = FALSE;
-      }
-      break;
-    default:
-      break;
-  }
-
-  if (forward)
-    ret = gst_pad_push_event (rtpsession->recv_rtp_sink, event);
-
-  gst_object_unref (rtpsession);
-
-  return ret;
-}
-
-
 static GstIterator *
 gst_rtp_session_iterate_internal_links (GstPad * pad)
 {
   GstRtpSession *rtpsession;
   GstPad *otherpad = NULL;
-  GstIterator *it = NULL;
+  GstIterator *it;
 
   rtpsession = GST_RTP_SESSION (gst_pad_get_parent (pad));
-  if (G_UNLIKELY (rtpsession == NULL))
-    return NULL;
 
-  GST_RTP_SESSION_LOCK (rtpsession);
   if (pad == rtpsession->recv_rtp_src) {
-    otherpad = gst_object_ref (rtpsession->recv_rtp_sink);
+    otherpad = rtpsession->recv_rtp_sink;
   } else if (pad == rtpsession->recv_rtp_sink) {
-    otherpad = gst_object_ref (rtpsession->recv_rtp_src);
+    otherpad = rtpsession->recv_rtp_src;
   } else if (pad == rtpsession->send_rtp_src) {
-    otherpad = gst_object_ref (rtpsession->send_rtp_sink);
+    otherpad = rtpsession->send_rtp_sink;
   } else if (pad == rtpsession->send_rtp_sink) {
-    otherpad = gst_object_ref (rtpsession->send_rtp_src);
+    otherpad = rtpsession->send_rtp_src;
   }
-  GST_RTP_SESSION_UNLOCK (rtpsession);
 
-  if (otherpad) {
-    it = gst_iterator_new_single (GST_TYPE_PAD, otherpad,
-        (GstCopyFunction) gst_object_ref, (GFreeFunc) gst_object_unref);
-    gst_object_unref (otherpad);
-  }
+  it = gst_iterator_new_single (GST_TYPE_PAD, otherpad,
+      (GstCopyFunction) gst_object_ref, (GFreeFunc) gst_object_unref);
 
   gst_object_unref (rtpsession);
 
@@ -1590,7 +1475,7 @@ gst_rtp_session_chain_recv_rtcp (GstPad * pad, GstBuffer * buffer)
   GstRtpSession *rtpsession;
   GstRtpSessionPrivate *priv;
   GstClockTime current_time;
-  guint64 ntpnstime;
+  GstFlowReturn ret;
 
   rtpsession = GST_RTP_SESSION (gst_pad_get_parent (pad));
   priv = rtpsession->priv;
@@ -1598,13 +1483,11 @@ gst_rtp_session_chain_recv_rtcp (GstPad * pad, GstBuffer * buffer)
   GST_LOG_OBJECT (rtpsession, "received RTCP packet");
 
   current_time = gst_clock_get_time (priv->sysclock);
-  get_current_times (rtpsession, NULL, &ntpnstime);
-
-  rtp_session_process_rtcp (priv->session, buffer, current_time, ntpnstime);
+  ret = rtp_session_process_rtcp (priv->session, buffer, current_time);
 
   gst_object_unref (rtpsession);
 
-  return GST_FLOW_OK;           /* always return OK */
+  return GST_FLOW_OK;
 }
 
 static gboolean
@@ -1637,13 +1520,10 @@ static gboolean
 gst_rtp_session_event_send_rtcp_src (GstPad * pad, GstEvent * event)
 {
   GstRtpSession *rtpsession;
-  gboolean ret = TRUE;
+  gboolean ret;
 
   rtpsession = GST_RTP_SESSION (gst_pad_get_parent (pad));
-  if (G_UNLIKELY (rtpsession == NULL)) {
-    gst_event_unref (event);
-    return FALSE;
-  }
+
   GST_DEBUG_OBJECT (rtpsession, "received EVENT");
 
   switch (GST_EVENT_TYPE (event)) {
@@ -1660,6 +1540,7 @@ gst_rtp_session_event_send_rtcp_src (GstPad * pad, GstEvent * event)
   }
 
   gst_object_unref (rtpsession);
+
   return ret;
 }
 
@@ -1722,21 +1603,9 @@ gst_rtp_session_event_send_rtp_sink (GstPad * pad, GstEvent * event)
           current_time);
       break;
     }
-    default:{
-      GstPad *send_rtp_src = NULL;
-      GST_RTP_SESSION_LOCK (rtpsession);
-      if (rtpsession->send_rtp_src)
-        send_rtp_src = gst_object_ref (rtpsession->send_rtp_src);
-      GST_RTP_SESSION_UNLOCK (rtpsession);
-
-      if (send_rtp_src) {
-        ret = gst_pad_push_event (send_rtp_src, event);
-        gst_object_unref (send_rtp_src);
-      } else
-        gst_event_unref (event);
-
+    default:
+      ret = gst_pad_push_event (rtpsession->send_rtp_src, event);
       break;
-    }
   }
   gst_object_unref (rtpsession);
 
@@ -1895,8 +1764,6 @@ create_recv_rtp_sink (GstRtpSession * rtpsession)
   rtpsession->recv_rtp_src =
       gst_pad_new_from_static_template (&rtpsession_recv_rtp_src_template,
       "recv_rtp_src");
-  gst_pad_set_event_function (rtpsession->recv_rtp_src,
-      (GstPadEventFunction) gst_rtp_session_event_recv_rtp_src);
   gst_pad_set_iterate_internal_links_function (rtpsession->recv_rtp_src,
       gst_rtp_session_iterate_internal_links);
   gst_pad_use_fixed_caps (rtpsession->recv_rtp_src);
@@ -2174,24 +2041,3 @@ wrong_pad:
     return;
   }
 }
-
-static void
-gst_rtp_session_request_key_unit (RTPSession * sess,
-    gboolean all_headers, gpointer user_data)
-{
-  GstRtpSession *rtpsession = GST_RTP_SESSION (user_data);
-  GstEvent *event;
-
-  event = gst_event_new_custom (GST_EVENT_CUSTOM_UPSTREAM,
-      gst_structure_new ("GstForceKeyUnit",
-          "all-headers", G_TYPE_BOOLEAN, all_headers, NULL));
-  gst_pad_push_event (rtpsession->send_rtp_sink, event);
-}
-
-static GstClockTime
-gst_rtp_session_request_time (RTPSession * session, gpointer user_data)
-{
-  GstRtpSession *rtpsession = GST_RTP_SESSION (user_data);
-
-  return gst_clock_get_time (rtpsession->priv->sysclock);
-}
