@@ -123,6 +123,76 @@ rtp_source_class_init (RTPSourceClass * klass)
    * The statistics of the source. This property returns a GstStructure with
    * name application/x-rtp-source-stats with the following fields:
    *
+   *  "ssrc"         G_TYPE_UINT     The SSRC of this source
+   *  "internal"     G_TYPE_BOOLEAN  If this source is the source of the session
+   *  "validated"    G_TYPE_BOOLEAN  If the source is validated
+   *  "received-bye" G_TYPE_BOOLEAN  If we received a BYE from this source
+   *  "is-csrc"      G_TYPE_BOOLEAN  If this source was found as CSRC
+   *  "is-sender"    G_TYPE_BOOLEAN  If this source is a sender
+   *  "seqnum-base"  G_TYPE_INT      first seqnum if known
+   *  "clock-rate"   G_TYPE_INT      the clock rate of the media
+   *
+   * The following two fields are only present when known.
+   *
+   *  "rtp-from"     G_TYPE_STRING   where we received the last RTP packet from
+   *  "rtcp-from"    G_TYPE_STRING   where we received the last RTCP packet from
+   *
+   * The following fields make sense for internal sources and will only increase
+   * when "is-sender" is TRUE:
+   *
+   *  "octets-sent"  G_TYPE_UINT64   number of bytes we sent
+   *  "packets-sent" G_TYPE_UINT64   number of packets we sent
+   *
+   * The following fields make sense for non-internal sources and will only
+   * increase when "is-sender" is TRUE.
+   *
+   *  "octets-received"  G_TYPE_UINT64  total number of bytes received
+   *  "packets-received" G_TYPE_UINT64  total number of packets received
+   *
+   * Following fields are updated when "is-sender" is TRUE.
+   *
+   *  "bitrate"      G_TYPE_UINT64   bitrate in bits per second
+   *  "jitter"       G_TYPE_UINT     estimated jitter
+   *  "packets-lost" G_TYPE_INT      estimated amount of packets lost
+   *
+   * The last SR report this source sent. This only updates when "is-sender" is
+   * TRUE.
+   *
+   *  "have-sr"         G_TYPE_BOOLEAN  the source has sent SR
+   *  "sr-ntptime"      G_TYPE_UINT64   ntptime of SR
+   *  "sr-rtptime"      G_TYPE_UINT     rtptime of SR
+   *  "sr-octet-count"  G_TYPE_UINT     the number of bytes in the SR
+   *  "sr-packet-count" G_TYPE_UINT     the number of packets in the SR
+   *
+   * The following fields are only present for non-internal sources and
+   * represent the content of the last RB packet that was sent to this source.
+   * These values are only updated when the source is sending.
+   *
+   *  "sent-rb"               G_TYPE_BOOLEAN  we have sent an RB
+   *  "sent-rb-fractionlost"  G_TYPE_UINT     calculated lost fraction
+   *  "sent-rb-packetslost"   G_TYPE_INT      lost packets
+   *  "sent-rb-exthighestseq" G_TYPE_UINT     last seen seqnum
+   *  "sent-rb-jitter"        G_TYPE_UINT     jitter
+   *  "sent-rb-lsr"           G_TYPE_UINT     last SR time
+   *  "sent-rb-dlsr"          G_TYPE_UINT     delay since last SR
+   *
+   * The following fields are only present for non-internal sources and
+   * represents the last RB that this source sent. This is only updated
+   * when the source is receiving data and sending RB blocks.
+   *
+   *  "have-rb"          G_TYPE_BOOLEAN  the source has sent RB
+   *  "rb-fractionlost"  G_TYPE_UINT     lost fraction
+   *  "rb-packetslost"   G_TYPE_INT      lost packets
+   *  "rb-exthighestseq" G_TYPE_UINT     highest received seqnum
+   *  "rb-jitter"        G_TYPE_UINT     reception jitter
+   *  "rb-lsr"           G_TYPE_UINT     last SR time
+   *  "rb-dlsr"          G_TYPE_UINT     delay since last SR
+   *
+   * The round trip of this source. This is calculated from the last RB
+   * values and the recption time of the last RB packet. Only present for
+   * non-internal sources.
+   *
+   *  "rb-round-trip"    G_TYPE_UINT     the round trip time in nanoseconds
    */
   g_object_class_install_property (gobject_class, PROP_STATS,
       g_param_spec_boxed ("stats", "Stats",
@@ -158,6 +228,7 @@ rtp_source_init (RTPSource * src)
   src->validated = FALSE;
   src->internal = FALSE;
   src->probation = RTP_DEFAULT_PROBATION;
+  src->closing = FALSE;
 
   src->sdes = gst_structure_new ("application/x-rtp-source-sdes", NULL);
 
@@ -188,6 +259,9 @@ rtp_source_finalize (GObject * object)
 
   gst_caps_replace (&src->caps, NULL);
 
+  g_list_foreach (src->conflicting_addresses, (GFunc) g_free, NULL);
+  g_list_free (src->conflicting_addresses);
+
   G_OBJECT_CLASS (rtp_source_parent_class)->finalize (object);
 }
 
@@ -198,6 +272,21 @@ rtp_source_create_stats (RTPSource * src)
   gboolean is_sender = src->is_sender;
   gboolean internal = src->internal;
   gchar address_str[GST_NETADDRESS_MAX_LEN];
+  gboolean have_rb;
+  guint8 fractionlost = 0;
+  gint32 packetslost = 0;
+  guint32 exthighestseq = 0;
+  guint32 jitter = 0;
+  guint32 lsr = 0;
+  guint32 dlsr = 0;
+  guint32 round_trip = 0;
+  gboolean have_sr;
+  GstClockTime time = 0;
+  guint64 ntptime = 0;
+  guint32 rtptime = 0;
+  guint32 packet_count = 0;
+  guint32 octet_count = 0;
+
 
   /* common data for all types of sources */
   s = gst_structure_new ("application/x-rtp-source-stats",
@@ -206,7 +295,9 @@ rtp_source_create_stats (RTPSource * src)
       "validated", G_TYPE_BOOLEAN, src->validated,
       "received-bye", G_TYPE_BOOLEAN, src->received_bye,
       "is-csrc", G_TYPE_BOOLEAN, src->is_csrc,
-      "is-sender", G_TYPE_BOOLEAN, is_sender, NULL);
+      "is-sender", G_TYPE_BOOLEAN, is_sender,
+      "seqnum-base", G_TYPE_INT, src->seqnum_base,
+      "clock-rate", G_TYPE_INT, src->clock_rate, NULL);
 
   /* add address and port */
   if (src->have_rtp_from) {
@@ -220,52 +311,39 @@ rtp_source_create_stats (RTPSource * src)
     gst_structure_set (s, "rtcp-from", G_TYPE_STRING, address_str, NULL);
   }
 
-  if (internal) {
-    /* our internal source */
-    if (is_sender) {
-      /* if we are sending, report about how much we sent, other sources will
-       * have a RB with info on reception. */
-      gst_structure_set (s,
-          "octets-sent", G_TYPE_UINT64, src->stats.octets_sent,
-          "packets-sent", G_TYPE_UINT64, src->stats.packets_sent,
-          "bitrate", G_TYPE_UINT64, src->bitrate, NULL);
-    } else {
-      /* if we are not sending we have nothing more to report */
-    }
-  } else {
-    gboolean have_rb;
-    guint8 fractionlost = 0;
-    gint32 packetslost = 0;
-    guint32 exthighestseq = 0;
-    guint32 jitter = 0;
-    guint32 lsr = 0;
-    guint32 dlsr = 0;
-    guint32 round_trip = 0;
+  gst_structure_set (s,
+      "octets-sent", G_TYPE_UINT64, src->stats.octets_sent,
+      "packets-sent", G_TYPE_UINT64, src->stats.packets_sent,
+      "octets-received", G_TYPE_UINT64, src->stats.octets_received,
+      "packets-received", G_TYPE_UINT64, src->stats.packets_received,
+      "bitrate", G_TYPE_UINT64, src->bitrate,
+      "packets-lost", G_TYPE_INT,
+      (gint) rtp_stats_get_packets_lost (&src->stats), "jitter", G_TYPE_UINT,
+      (guint) (src->stats.jitter >> 4), NULL);
 
-    /* other sources */
-    if (is_sender) {
-      gboolean have_sr;
-      GstClockTime time = 0;
-      guint64 ntptime = 0;
-      guint32 rtptime = 0;
-      guint32 packet_count = 0;
-      guint32 octet_count = 0;
+  /* get the last SR. */
+  have_sr = rtp_source_get_last_sr (src, &time, &ntptime, &rtptime,
+      &packet_count, &octet_count);
+  gst_structure_set (s,
+      "have-sr", G_TYPE_BOOLEAN, have_sr,
+      "sr-ntptime", G_TYPE_UINT64, ntptime,
+      "sr-rtptime", G_TYPE_UINT, (guint) rtptime,
+      "sr-octet-count", G_TYPE_UINT, (guint) octet_count,
+      "sr-packet-count", G_TYPE_UINT, (guint) packet_count, NULL);
 
-      /* this source is sending to us, get the last SR. */
-      have_sr = rtp_source_get_last_sr (src, &time, &ntptime, &rtptime,
-          &packet_count, &octet_count);
-      gst_structure_set (s,
-          "octets-received", G_TYPE_UINT64, src->stats.octets_received,
-          "packets-received", G_TYPE_UINT64, src->stats.packets_received,
-          "bitrate", G_TYPE_UINT64, src->bitrate,
-          "have-sr", G_TYPE_BOOLEAN, have_sr,
-          "sr-ntptime", G_TYPE_UINT64, ntptime,
-          "sr-rtptime", G_TYPE_UINT, (guint) rtptime,
-          "sr-octet-count", G_TYPE_UINT, (guint) octet_count,
-          "sr-packet-count", G_TYPE_UINT, (guint) packet_count, NULL);
-    }
-    /* we might be sending to this SSRC so we report about how it is
-     * receiving our data */
+  if (!internal) {
+    /* get the last RB we sent */
+    gst_structure_set (s,
+        "sent-rb", G_TYPE_BOOLEAN, src->last_rr.is_valid,
+        "sent-rb-fractionlost", G_TYPE_UINT, (guint) src->last_rr.fractionlost,
+        "sent-rb-packetslost", G_TYPE_INT, (gint) src->last_rr.packetslost,
+        "sent-rb-exthighestseq", G_TYPE_UINT,
+        (guint) src->last_rr.exthighestseq, "sent-rb-jitter", G_TYPE_UINT,
+        (guint) src->last_rr.jitter, "sent-rb-lsr", G_TYPE_UINT,
+        (guint) src->last_rr.lsr, "sent-rb-dlsr", G_TYPE_UINT,
+        (guint) src->last_rr.dlsr, NULL);
+
+    /* get the last RB */
     have_rb = rtp_source_get_last_rb (src, &fractionlost, &packetslost,
         &exthighestseq, &jitter, &lsr, &dlsr, &round_trip);
 
@@ -886,6 +964,8 @@ init_seq (RTPSource * src, guint16 seq)
   GST_DEBUG ("base_seq %d", seq);
 }
 
+#define BITRATE_INTERVAL (2 * GST_SECOND)
+
 static void
 do_bitrate_estimation (RTPSource * src, GstClockTime running_time,
     guint64 * bytes_handled)
@@ -895,12 +975,10 @@ do_bitrate_estimation (RTPSource * src, GstClockTime running_time,
   if (src->prev_rtime) {
     elapsed = running_time - src->prev_rtime;
 
-    if (elapsed > (G_GINT64_CONSTANT (1) << 31)) {
+    if (elapsed > BITRATE_INTERVAL) {
       guint64 rate;
 
-      rate =
-          gst_util_uint64_scale (*bytes_handled, elapsed,
-          (G_GINT64_CONSTANT (1) << 29));
+      rate = gst_util_uint64_scale (*bytes_handled, 8 * GST_SECOND, elapsed);
 
       GST_LOG ("Elapsed %" G_GUINT64_FORMAT ", bytes %" G_GUINT64_FORMAT
           ", rate %" G_GUINT64_FORMAT, elapsed, *bytes_handled, rate);
@@ -1582,3 +1660,88 @@ rtp_source_get_last_rb (RTPSource * src, guint8 * fractionlost,
 
   return TRUE;
 }
+
+/**
+ * rtp_source_find_conflicting_address:
+ * @src: The source the packet came in
+ * @address: address to check for
+ * @time: The time when the packet that is possibly in conflict arrived
+ *
+ * Checks if an address which has a conflict is already known. If it is
+ * a known conflict, remember the time
+ *
+ * Returns: TRUE if it was a known conflict, FALSE otherwise
+ */
+gboolean
+rtp_source_find_conflicting_address (RTPSource * src, GstNetAddress * address,
+    GstClockTime time)
+{
+  GList *item;
+
+  for (item = g_list_first (src->conflicting_addresses);
+      item; item = g_list_next (item)) {
+    RTPConflictingAddress *known_conflict = item->data;
+
+    if (gst_netaddress_equal (address, &known_conflict->address)) {
+      known_conflict->time = time;
+      return TRUE;
+    }
+  }
+
+  return FALSE;
+}
+
+/**
+ * rtp_source_add_conflicting_address:
+ * @src: The source the packet came in
+ * @address: address to remember
+ * @time: The time when the packet that is in conflict arrived
+ *
+ * Adds a new conflict address
+ */
+void
+rtp_source_add_conflicting_address (RTPSource * src,
+    GstNetAddress * address, GstClockTime time)
+{
+  RTPConflictingAddress *new_conflict;
+
+  new_conflict = g_new0 (RTPConflictingAddress, 1);
+
+  memcpy (&new_conflict->address, address, sizeof (GstNetAddress));
+  new_conflict->time = time;
+
+  src->conflicting_addresses = g_list_prepend (src->conflicting_addresses,
+      new_conflict);
+}
+
+/**
+ * rtp_source_timeout:
+ * @src: The #RTPSource
+ * @current_time: The current time
+ * @collision_timeout: The amount of time after which a collision is timed out
+ *
+ * This is processed on each RTCP interval. It times out old collisions.
+ */
+void
+rtp_source_timeout (RTPSource * src, GstClockTime current_time,
+    GstClockTime collision_timeout)
+{
+  GList *item;
+
+  item = g_list_first (src->conflicting_addresses);
+  while (item) {
+    RTPConflictingAddress *known_conflict = item->data;
+    GList *next_item = g_list_next (item);
+
+    if (known_conflict->time < current_time - collision_timeout) {
+      gchar buf[40];
+
+      src->conflicting_addresses =
+          g_list_delete_link (src->conflicting_addresses, item);
+      gst_netaddress_to_string (&known_conflict->address, buf, 40);
+      GST_DEBUG ("collision %p timed out: %s", known_conflict, buf);
+      g_free (known_conflict);
+    }
+    item = next_item;
+  }
+}
