@@ -69,7 +69,8 @@ enum
   PROP_STARTX,
   PROP_STARTY,
   PROP_ENDX,
-  PROP_ENDY
+  PROP_ENDY,
+  PROP_REMOTE,
 };
 
 GST_BOILERPLATE (GstXImageSrc, gst_ximage_src, GstPushSrc, GST_TYPE_PUSH_SRC);
@@ -389,7 +390,8 @@ gst_ximage_src_ximage_get (GstXImageSrc * ximagesrc)
         (BufferReturnFunc) (gst_ximage_src_return_buf));
     if (ximage == NULL) {
       GST_ELEMENT_ERROR (ximagesrc, RESOURCE, WRITE, (NULL),
-          ("could not create a %dx%d ximage", ximage->width, ximage->height));
+          ("could not create a %dx%d ximage", ximagesrc->width,
+              ximagesrc->height));
       g_mutex_unlock (ximagesrc->x_lock);
       return NULL;
     }
@@ -464,10 +466,10 @@ gst_ximage_src_ximage_get (GstXImageSrc * ximagesrc)
             if (ximagesrc->endx > ximagesrc->startx &&
                 ximagesrc->endy > ximagesrc->starty) {
               /* see if damage area intersects */
-              if (rects[i].x + rects[i].width < ximagesrc->startx ||
+              if (rects[i].x + rects[i].width - 1 < ximagesrc->startx ||
                   rects[i].x > ximagesrc->endx) {
                 /* trivial reject */
-              } else if (rects[i].y + rects[i].height < ximagesrc->starty ||
+              } else if (rects[i].y + rects[i].height - 1 < ximagesrc->starty ||
                   rects[i].y > ximagesrc->endy) {
                 /* trivial reject */
               } else {
@@ -478,12 +480,12 @@ gst_ximage_src_ximage_get (GstXImageSrc * ximagesrc)
                     rects[i].x;
                 starty = (rects[i].y < ximagesrc->starty) ? ximagesrc->starty :
                     rects[i].y;
-                width = (rects[i].x + rects[i].width < ximagesrc->endx) ?
+                width = (rects[i].x + rects[i].width - 1 < ximagesrc->endx) ?
                     rects[i].x + rects[i].width - startx :
-                    ximagesrc->endx - startx;
-                height = (rects[i].y + rects[i].height < ximagesrc->endy) ?
+                    ximagesrc->endx - startx + 1;
+                height = (rects[i].y + rects[i].height - 1 < ximagesrc->endy) ?
                     rects[i].y + rects[i].height - starty : ximagesrc->endy -
-                    starty;
+                    starty + 1;
 
                 GST_LOG_OBJECT (ximagesrc,
                     "Retrieving damaged sub-region @ %d,%d size %dx%d as intersect region",
@@ -590,9 +592,16 @@ gst_ximage_src_ximage_get (GstXImageSrc * ximagesrc)
 #endif /* HAVE_XSHM */
     {
       GST_DEBUG_OBJECT (ximagesrc, "Retrieving screen using XGetImage");
-      ximage->ximage = XGetImage (ximagesrc->xcontext->disp, ximagesrc->xwindow,
-          ximagesrc->startx, ximagesrc->starty, ximagesrc->width,
-          ximagesrc->height, AllPlanes, ZPixmap);
+      if (ximagesrc->remote) {
+        XGetSubImage (ximagesrc->xcontext->disp, ximagesrc->xwindow,
+            ximagesrc->startx, ximagesrc->starty, ximagesrc->width,
+            ximagesrc->height, AllPlanes, ZPixmap, ximage->ximage, 0, 0);
+      } else {
+        ximage->ximage =
+            XGetImage (ximagesrc->xcontext->disp, ximagesrc->xwindow,
+            ximagesrc->startx, ximagesrc->starty, ximagesrc->width,
+            ximagesrc->height, AllPlanes, ZPixmap);
+      }
     }
 #ifdef HAVE_XDAMAGE
   }
@@ -823,6 +832,9 @@ gst_ximage_src_set_property (GObject * object, guint prop_id,
     case PROP_ENDY:
       src->endy = g_value_get_uint (value);
       break;
+    case PROP_REMOTE:
+      src->remote = g_value_get_boolean (value);
+      break;
     default:
       break;
   }
@@ -863,6 +875,9 @@ gst_ximage_src_get_property (GObject * object, guint prop_id, GValue * value,
     case PROP_ENDY:
       g_value_set_uint (value, src->endy);
       break;
+    case PROP_REMOTE:
+      g_value_set_boolean (value, src->remote);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -942,27 +957,36 @@ gst_ximage_src_get_caps (GstBaseSrc * bs)
 
   width = xcontext->width;
   height = xcontext->height;
-  if (s->endx > s->startx && s->endy > s->starty) {
+
+  /* property comments say 0 means right/bottom, means we can't capture
+     the top left pixel alone */
+  if (s->endx == 0)
+    s->endx = width - 1;
+  if (s->endy == 0)
+    s->endy = height - 1;
+
+  if (s->endx >= s->startx && s->endy >= s->starty) {
     /* this means user has put in values */
     if (s->startx < xcontext->width && s->endx < xcontext->width &&
-        s->starty < xcontext->height && s->endy < xcontext->height) {
+        s->starty < xcontext->height && s->endy < xcontext->height &&
+        s->startx >= 0 && s->starty >= 0) {
       /* values are fine */
-      s->width = width = s->endx - s->startx;
-      s->height = height = s->endy - s->starty;
+      s->width = width = s->endx - s->startx + 1;
+      s->height = height = s->endy - s->starty + 1;
     } else {
       GST_WARNING
           ("User put in co-ordinates overshooting the X resolution, setting to full screen");
       s->startx = 0;
       s->starty = 0;
-      s->endx = 0;
-      s->endy = 0;
+      s->endx = width - 1;
+      s->endy = height - 1;
     }
   } else {
     GST_WARNING ("User put in bogus co-ordinates, setting to full screen");
     s->startx = 0;
     s->starty = 0;
-    s->endx = 0;
-    s->endy = 0;
+    s->endx = width - 1;
+    s->endy = height - 1;
   }
   GST_DEBUG ("width = %d, height=%d", width, height);
   return gst_caps_new_simple ("video/x-raw-rgb",
@@ -1032,14 +1056,14 @@ gst_ximage_src_class_init (GstXImageSrcClass * klass)
 
   g_object_class_install_property (gc, PROP_DISPLAY_NAME,
       g_param_spec_string ("display-name", "Display", "X Display Name", NULL,
-          G_PARAM_READWRITE));
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (gc, PROP_SCREEN_NUM,
       g_param_spec_uint ("screen-num", "Screen number", "X Screen Number",
-          0, G_MAXINT, 0, G_PARAM_READWRITE));
+          0, G_MAXINT, 0, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (gc, PROP_SHOW_POINTER,
       g_param_spec_boolean ("show-pointer", "Show Mouse Pointer",
           "Show mouse pointer (if XFixes extension enabled)", TRUE,
-          G_PARAM_READWRITE));
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   /**
    * GstXImageSrc:use-damage
    *
@@ -1050,7 +1074,7 @@ gst_ximage_src_class_init (GstXImageSrcClass * klass)
   g_object_class_install_property (gc, PROP_USE_DAMAGE,
       g_param_spec_boolean ("use-damage", "Use XDamage",
           "Use XDamage (if XDamage extension enabled)", TRUE,
-          G_PARAM_READWRITE));
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   /**
    * GstXImageSrc:startx
    *
@@ -1062,7 +1086,7 @@ gst_ximage_src_class_init (GstXImageSrcClass * klass)
   g_object_class_install_property (gc, PROP_STARTX,
       g_param_spec_uint ("startx", "Start X co-ordinate",
           "X coordinate of top left corner of area to be recorded (0 for top left of screen)",
-          0, G_MAXINT, 0, G_PARAM_READWRITE));
+          0, G_MAXINT, 0, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   /**
    * GstXImageSrc:starty
    *
@@ -1100,6 +1124,19 @@ gst_ximage_src_class_init (GstXImageSrcClass * klass)
           "Y coordinate of bottom right corner of area to be recorded (0 for bottom right of screen)",
           0, G_MAXINT, 0, G_PARAM_READWRITE));
 
+  /**
+   * GstXImageSrc:remote
+   *
+   * Whether the X display is remote. The element will try to use alternate calls
+   * known to work better with remote displays.
+   *
+   * Since: 0.10.26
+   **/
+  g_object_class_install_property (gc, PROP_REMOTE,
+      g_param_spec_boolean ("remote", "Remote dispay",
+          "Whether the display is remote", FALSE,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+
   parent_class = g_type_class_peek_parent (klass);
 
   push_class->create = gst_ximage_src_create;
@@ -1126,6 +1163,7 @@ gst_ximage_src_init (GstXImageSrc * ximagesrc, GstXImageSrcClass * klass)
   ximagesrc->starty = 0;
   ximagesrc->endx = 0;
   ximagesrc->endy = 0;
+  ximagesrc->remote = FALSE;
 }
 
 static gboolean
